@@ -11,13 +11,41 @@ use Sub::Infix qw(infix);
 
 BEGIN {
 	$match::simple::AUTHORITY = 'cpan:TOBYINK';
-	$match::simple::VERSION   = '0.006';
+	$match::simple::VERSION   = '0.008';
 }
 
 our @ISA       = qw( Exporter::Tiny );
 our @EXPORT    = qw( M );
 our @EXPORT_OK = qw( match );
 
+my $xs;
+unless (($ENV{MATCH_SIMPLE_IMPLEMENTATION}||'') =~ /pp/i)
+{
+	eval {
+		require match::simple::XS;
+		match::simple::XS->VERSION(0.001);  # minimum
+		
+		# Unless we're a development version...
+		# Avoid using an unstable version of ::XS
+		unless (match::simple->VERSION =~ /_/)
+		{
+			die if match::simple::XS->VERSION =~ /_/;
+		}
+			
+		$xs = match::simple::XS->can('match');
+	};
+}
+
+eval($xs ? <<'XS' : <<'PP');
+
+sub IMPLEMENTATION () { "XS" }
+
+*match = *match::simple::XS::match;
+
+XS
+
+sub IMPLEMENTATION () { "PP" }
+
 sub match
 {
 	no warnings qw(uninitialized numeric);
@@ -37,6 +65,8 @@ sub match
 	Carp::croak("match::simple cannot match anything against: $b");
 }
 
+PP
+
 *M = &infix(\&match);
 
 __END__
@@ -124,12 +154,38 @@ export a more normal function:
       say "$this matches $that";
    }
 
+If you're making heavy use of this module, then this is probably your best
+option, as it runs significantly faster.
+
+=head2 XS Backend
+
+If you install match::simple::XS, a faster XS-based implementation will be
+used instead of the pure Perl functions. Depending on what sort of match you
+are doing, this is likely to be several times faster. In extreme cases, such
+as matching a string in an arrayref, it can be twenty-five times faster, or
+more. However, where C<< $that >> is a single regexp, it's around 30% slower.
+Overall though, I think the performance improvement is worthwhile.
+
+If you want to take advantage of this speed up, use the C<match> function
+rather than the C<< |M| >> operator. Otherwise all your gains will be lost to
+the slow implementation of operator overloading.
+
+The constant C<< match::simple::IMPLEMENTATION >> tells you which backend
+is currently in use.
+
+=head2 Environment
+
+Setting the C<MATCH_SIMPLE_IMPLEMENTATION> environment variable to "PP"
+encourages match::simple to use the pure Perl backend.
+
 =begin trustme
 
 =item M
 
 =item match
 
+=item IMPLEMENTATION
+
 =end trustme
 
 =head1 BUGS
@@ -149,7 +205,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013 by Toby Inkster.
+This software is copyright (c) 2013-2014 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
